<?php if (! $this->compact): ?>
<div class="controls separated">
    <?= $this->tabs ?>
    <?= $this->paginator ?>
    <div class="sort-controls-container">
        <?= $this->limiter ?>
    </div>
</div>
<?php endif ?>
<div class="content">
<?php if ($this->logData !== null): ?>
    <table class="action">
    <tbody>
    <?php foreach ($this->logData as $value): ?>
        <?php $datetime = new Datetime($value->datetime) ?>
        <tr>
            <td>
                <?= $this->escape($datetime->format('d.m. H:i')) ?><br />
                <?= $this->escape($value->loglevel) ?>
            </td>
            <td>
                <?= nl2br($this->escape($value->message), false) ?>
            </td>
        </tr>
    <?php endforeach ?>
    </tbody>
    </table>
<?php endif ?>
</div>
