<?php

// SPDX-FileCopyrightText: 2018 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

namespace Icinga\Exception;

/**
 * Exception thrown if something to add already exists
 */
class AlreadyExistsException extends IcingaException
{
}
