<?php

// SPDX-FileCopyrightText: 2018 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

namespace Icinga\Web\Form;

/**
 * Exceptions for invalid form tokens
 */
class InvalidCSRFTokenException extends \Exception
{
}
