<?php

// SPDX-FileCopyrightText: 2018 Icinga GmbH <https://icinga.com>
// SPDX-License-Identifier: GPL-3.0-or-later

namespace Icinga\Protocol\File;

use Icinga\Exception\IcingaException;

/**
 * Exception thrown if a file reader specific error occurs
 */
class FileReaderException extends IcingaException
{
}
